"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypeIs = exports.TypePaging = exports.TypeArray = exports.parseToSwagger = void 0;

var _sequelize = require("sequelize");

var _types = require("../types");

var parseToSwagger = function parseToSwagger(v) {
  if (!v) return undefined;
  return Object.keys(v).reduce(function (x, y) {
    var type = v[y].type;
    if ('function' === typeof type) type = type();

    if ('__dto_name' in type) {
      x[y] = {
        type: 'items',
        properties: parseToSwagger(type.properties)
      };
    } else {
      x[y] = type.toSwagger();
    }

    return x;
  }, {});
};

exports.parseToSwagger = parseToSwagger;

var TypeArray = function TypeArray(o) {
  return function () {
    var x = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      x[_i] = arguments[_i];
    }

    var tp = (0, _types.parseType)(o, x);
    return {
      __name: 'array',
      toSwagger: function toSwagger() {
        var _a, _b;

        return {
          type: 'array',
          items: tp.isDto ? {
            type: 'object',
            properties: parseToSwagger((_a = tp.dto) === null || _a === void 0 ? void 0 : _a.properties)
          } : (_b = tp.typeIs) === null || _b === void 0 ? void 0 : _b.toSwagger()
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.JSON;
      }
    };
  };
};

exports.TypeArray = TypeArray;

var TypePaging = function TypePaging(o) {
  var items = '__dto_name' in o ? {
    $ref: '#/components/schemas/' + o.__dto_name
  } : typeof o === 'function' ? o().toSwagger() : 'toSwagger' in o ? o.toSwagger() : {};
  return function () {
    return {
      __name: 'paging',
      toSwagger: function toSwagger() {
        return {
          type: 'object',
          properties: {
            page: {
              type: 'number',
              comment: '현재 페이지'
            },
            page_size: {
              type: 'number',
              comment: '페이지당 표시 갯수'
            },
            max_page: {
              type: 'number',
              comment: '최대페이지'
            },
            has_prev: {
              type: 'boolean',
              comment: '이전 이동가능 여부'
            },
            has_next: {
              type: 'boolean',
              comment: '다음 이동가능 여부'
            },
            total_elements: {
              type: 'number',
              comment: '전체 레코드개수'
            },
            items: {
              type: 'array',
              items: items
            }
          }
        };
      }
    };
  };
};

exports.TypePaging = TypePaging;
var TypeIs = {
  INT: function INT(options) {
    return {
      __name: 'int',
      toSwagger: function toSwagger() {
        return {
          type: 'integer',
          format: 'int32'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.INTEGER(options);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return parseInt(String(o), 10);
      }
    };
  },
  LONG: function LONG(options) {
    return {
      __name: 'long',
      toSwagger: function toSwagger() {
        return {
          type: 'integer',
          format: 'int64'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.INTEGER(options);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return parseInt(String(o), 10);
      }
    };
  },
  FLOAT: function FLOAT(length, decimals) {
    return {
      __name: 'float',
      toSwagger: function toSwagger() {
        return {
          type: 'integer',
          format: 'float'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.FLOAT(length, decimals);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return parseFloat(String(o));
      }
    };
  },
  DOUBLE: function DOUBLE(length, decimals) {
    return {
      __name: 'double',
      toSwagger: function toSwagger() {
        return {
          type: 'number',
          format: 'double'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.DOUBLE(length, decimals);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return parseFloat(String(o));
      }
    };
  },
  STRING: function STRING(length, binary) {
    return {
      __name: 'string',
      toSwagger: function toSwagger() {
        return {
          type: 'string'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.STRING(length, binary);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return String(o);
      }
    };
  },
  TEXT: function TEXT(length) {
    return {
      __name: 'text',
      toSwagger: function toSwagger() {
        return {
          type: 'string'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.TEXT({
          length: length
        });
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return String(o);
      }
    };
  },
  PASSWORD: function PASSWORD() {
    return {
      __name: 'password',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          format: 'password'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.STRING(512);
      }
    };
  },
  ENUM: function ENUM() {
    var texts = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      texts[_i] = arguments[_i];
    }

    return {
      __name: 'enum',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          "enum": texts,
          example: texts[0]
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.ENUM.apply(_sequelize.DataTypes, texts);
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return texts.includes(String(o)) ? String(o) : undefined;
      }
    };
  },
  JSON: function JSON() {
    return {
      __name: 'json',
      toSwagger: function toSwagger() {
        return {
          type: 'object'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.JSON;
      }
    };
  },
  BASE64: function BASE64(length) {
    return {
      __name: 'base64',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          format: 'byte'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.BLOB(length);
      }
    };
  },
  BINARY: function BINARY(length) {
    return {
      __name: 'binary',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          format: 'binary'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.BLOB(length);
      }
    };
  },
  BOOLEAN: function BOOLEAN() {
    return {
      __name: 'boolean',
      toSwagger: function toSwagger() {
        return {
          type: 'boolean'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.BOOLEAN;
      },
      fixValue: function fixValue(o) {
        if (o === undefined || o === null) return o;
        return String(o) === 'true' ? true : String(o) === 'false' ? false : !!o;
      }
    };
  },
  DATEONLY: function DATEONLY() {
    return {
      __name: 'date',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          format: 'date'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.DATEONLY();
      }
    };
  },
  DATETIME: function DATETIME() {
    return {
      __name: 'datetime',
      toSwagger: function toSwagger() {
        return {
          type: 'string',
          format: 'date-time'
        };
      },
      toSequelize: function toSequelize() {
        return _sequelize.DataTypes.DATE;
      }
    };
  }
};
exports.TypeIs = TypeIs;