"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDtoDefinitions = getDtoDefinitions;
exports.createDto = createDto;

var _tslib = require("tslib");

var _sequelize = require("sequelize");

var _types = require("../types");

var _service = require("./service");

var _database = require("./database");

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var config = {
  definitions: {}
};

function getDtoDefinitions() {
  return config.definitions;
}

function createDto(name, properties, entity) {
  var _a;

  config.definitions[name] = {
    type: 'object',
    properties: Object.keys(properties).reduce(function (p, key) {
      var _a, _b, _c;

      var property = properties[key];
      var tp = (0, _types.parseType)(property.type);

      if (tp.isDto) {
        p[key] = {
          $ref: '#/components/schemas/' + ((_a = tp.dto) === null || _a === void 0 ? void 0 : _a.__dto_name)
        };
      } else {
        var args = ((_c = (_b = tp.typeIs) === null || _b === void 0 ? void 0 : _b.toSwagger) === null || _c === void 0 ? void 0 : _c.call(_b)) || {};
        if (!args) args = {
          type: String(tp.typeIs)
        };
        args.description = property.comment;
        p[key] = args;
      }

      return p;
    }, {})
  };

  var map = function map(item) {
    var o = (item === null || item === void 0 ? void 0 : item.dataValues) || item;
    return Object.keys(properties).reduce(function (p, key) {
      var _a, _b, _c, _d;

      var property = properties[key];
      var tp = (0, _types.parseType)(property.type);

      if (tp === null || tp === void 0 ? void 0 : tp.isDto) {
        var k = "__" + property.reference;
        p[key] = (o === null || o === void 0 ? void 0 : o[k]) ? (_a = tp.dto) === null || _a === void 0 ? void 0 : _a.map(o[k]) : ((_b = tp.dto) === null || _b === void 0 ? void 0 : _b.map(o === null || o === void 0 ? void 0 : o[key])) || (o === null || o === void 0 ? void 0 : o[key]);
      } else if ((_c = tp === null || tp === void 0 ? void 0 : tp.typeIs) === null || _c === void 0 ? void 0 : _c.fixValue) {
        p[key] = (_d = tp === null || tp === void 0 ? void 0 : tp.typeIs) === null || _d === void 0 ? void 0 : _d.fixValue(o === null || o === void 0 ? void 0 : o[key]);
      } else {
        p[key] = o === null || o === void 0 ? void 0 : o[key];
      }

      if (property.render) {
        p[key] = property.render(p[key]);
      }

      if (property.secure && typeof p[key] === 'string') {
        p[key] = (0, _service.decodeAES128)(p[key], (0, _database.getSecureKey)());
      }

      return p;
    }, {});
  };

  var outProperties = Object.keys(properties).reduce(function (p, key) {
    p[key] = {
      // @ts-ignore
      type: properties[key].type
    };
    return p;
  }, {});
  var ATTRS = Object.keys(((_a = entity === null || entity === void 0 ? void 0 : entity.defineModel) === null || _a === void 0 ? void 0 : _a.rawAttributes) || {});
  return {
    __dto_name: name,
    defineModel: entity === null || entity === void 0 ? void 0 : entity.defineModel,
    properties: outProperties,
    map: map,
    collect: function collect(o) {
      return o === null || o === void 0 ? void 0 : o.map(function (v) {
        return map(v);
      });
    },
    middleware: function middleware(options, user) {
      var attrs = Object.keys(properties).reduce(function (x, y) {
        var _a, _b, _c, _d, _e, _f, _g, _h;

        var property = properties[y];
        var dbName = (_a = (0, _database.getSequelize)()) === null || _a === void 0 ? void 0 : _a.getDialect();
        var wrapper = '`';

        if (dbName === 'postgres') {
          wrapper = '"';
        }

        if (property) {
          if (property.query) {
            var association = ((options === null || options === void 0 ? void 0 : options.full_associations) ? wrapper + (options === null || options === void 0 ? void 0 : options.full_associations) + wrapper : options === null || options === void 0 ? void 0 : options.full_associations) || (options === null || options === void 0 ? void 0 : options.association) || ((_b = entity === null || entity === void 0 ? void 0 : entity.defineModel) === null || _b === void 0 ? void 0 : _b.tableName) || name;
            var query = 'function' === typeof property.query ? property.query({
              user: user,
              association: association,
              exportParams: options.exportParams
            }) : property.query;
            x.push([(0, _sequelize.literal)(query), y]);
            return x;
          }

          var tp = (0, _types.parseType)(property.type);

          if (tp.isDto) {
            if (!options.include) options.include = [];
            var association = "__" + ((property === null || property === void 0 ? void 0 : property.reference) || y);
            var associations = [options === null || options === void 0 ? void 0 : options.association, association].filter(function (v) {
              return !!v;
            });
            var where = undefined;

            if ((_c = options === null || options === void 0 ? void 0 : options.where) === null || _c === void 0 ? void 0 : _c[association]) {
              where = (0, _tslib.__assign)({}, (_d = options === null || options === void 0 ? void 0 : options.where) === null || _d === void 0 ? void 0 : _d[association]);
              (_e = options === null || options === void 0 ? void 0 : options.where) === null || _e === void 0 ? true : delete _e[association];
            }

            options.include.push((_f = tp.dto) === null || _f === void 0 ? void 0 : _f.middleware({
              model: (_g = tp.dto) === null || _g === void 0 ? void 0 : _g.defineModel,
              where: where,
              required: !!((options === null || options === void 0 ? void 0 : options.required) || where && Object.keys(where).length > 0 && ((_h = tp.dto) === null || _h === void 0 ? void 0 : _h.defineModel)),
              association: association,
              full_associations: associations.join('->'),
              as: y
            }, user));
          }

          x.push(property.column || y);
        }

        return x;
      }, []);
      attrs = attrs.filter(function (item, index) {
        return attrs.indexOf(item) === index;
      });
      if (!options.attributes) options.attributes = [];
      options.attributes = options.attributes.concat(attrs.filter(function (v) {
        return _typeof(v) === 'object' || ATTRS.includes(v);
      }));
      return options;
    }
  };
}