/// <reference types="passport" />
import { TypeIsDefine } from '../contants/TypeIs';
import { ValueObject } from './dto';
import { AuthOption } from '../types';
import express from 'express';
interface AppUser extends Express.User {
    id?: number;
    roles?: string[];
}
export interface RequestParam {
    [key: string]: TypeIsDefine;
}
export interface RoutePath {
    path: string;
    items?: {
        [key: string]: TypeIsDefine;
    };
}
export interface RouteRequest {
    uri: string | RoutePath;
    method: 'GET' | 'POST' | 'PUT' | 'DELETE';
    roles?: string[];
    query?: RequestParam;
    body?: RequestParam | ValueObject;
    formdata?: boolean;
    response?: ValueObject | (() => ValueObject) | TypeIsDefine | (() => TypeIsDefine);
}
export interface RouteOption {
    tags?: string[];
    summary?: string;
    description?: string;
    paging?: 'sort' | 'page' | true | false;
}
interface ExecuteArgs {
    request: unknown;
    path?: {
        [key: string]: unknown;
    };
    query: {
        [key: string]: unknown;
    };
    body: {
        [key: string]: unknown;
    };
    files?: {
        [key: string]: unknown;
    };
    user?: AppUser;
}
declare type ExecuteFunction = ((args: ExecuteArgs) => Promise<unknown> | unknown) & ((req: any, res: any, next: any) => Promise<unknown> | unknown | void);
export interface SwaggerData {
    operationId: string;
    path: RoutePath;
    method: 'GET' | 'POST' | 'PUT' | 'DELETE';
    tags?: string[];
    summary?: string;
    description?: string;
    parameters?: any[];
    requestBody?: {
        [key: string]: any;
    };
    response?: ValueObject | TypeIsDefine;
    data?: any;
}
export declare function getPages(): SwaggerData[];
export declare const createRoute: (request: RouteRequest, execute: ExecuteFunction, options: RouteOption) => {};
export declare const getJwtToken: (req: express.Request, user: object, refreshPayload: object) => Promise<unknown>;
export declare const installRoutes: (options: AuthOption) => Promise<import("express-serve-static-core").Router>;
export {};
