import { FindAndCountOptions, Model, ModelCtor } from 'sequelize/types/lib/model';
import { ValueObject } from './dto';
import { Transaction } from 'sequelize';
export declare function encodeAES128(str: string, key: string): string;
export declare function decodeAES128(str: string, key: string): string;
export declare type RepoFn = {
    findAll: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<T[]>;
    findOne: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<T>;
    findAndCountAll: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<{
        count: number;
        rows: T[];
    }>;
    count: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<{
        [key: string]: number;
    }>;
    create: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<T>;
    getObjects: <T extends Model>(model: ModelCtor<T>, args: GetObjectArgs) => Promise<any>;
    save: <T extends Model>(model: T) => Promise<T>;
    destroy: <T extends Model>(model: T) => Promise<void>;
    updateAll: <T extends Model>(model: ModelCtor<T>, values: {
        [key: string]: any;
    }, args: any) => Promise<any>;
    destroyAll: <T extends Model>(model: ModelCtor<T>, args: any) => Promise<any>;
};
export declare type ServiceMethodItem = (repo: RepoFn, args: unknown[], options?: {
    transaction?: Transaction;
}) => Promise<void>;
export declare type ServiceMethod<T> = {
    [P in keyof T]?: ServiceMethodItem;
};
export declare type ExportMethodType = (...args: unknown[]) => Promise<void>;
export declare type RunningMethod<T> = {
    [P in keyof T]?: ExportMethodType;
};
export interface PageRequest {
    page: number;
    limit: number;
    sort: string;
    dir: string;
}
export declare const pagingRequestParse: (params: {
    page?: any;
    limit?: any;
    sort?: any;
    dir?: any;
} | undefined) => PageRequest;
export declare const pagingResponseParse: (request: PageRequest, pageObject: {
    rows: any[];
    count: number;
}, fn: ((o: any) => any) | undefined) => {
    items: any[];
    page: number;
    page_size: number;
    max_page: number;
    has_prev: boolean;
    has_next: boolean;
    total_elements: number;
};
export interface GetObjectArgs extends FindAndCountOptions {
    pagable?: any;
    exportTo?: ValueObject;
    exportParams?: {
        [key: string]: unknown;
    };
    user?: any;
}
export declare const createService: <T extends ServiceMethod<T>>(properties: T) => RunningMethod<T>;
