"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.encodeAES128 = encodeAES128;
exports.decodeAES128 = decodeAES128;
exports.createService = exports.pagingResponseParse = exports.pagingRequestParse = void 0;

var _tslib = require("tslib");

var _database = require("./database");

var _sequelize = require("sequelize");

var _cryptoJs = _interopRequireDefault(require("crypto-js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function encodeAES128(str, key) {
  var cipher = _cryptoJs["default"].AES.encrypt(str, _cryptoJs["default"].enc.Utf8.parse(key), {
    iv: _cryptoJs["default"].enc.Utf8.parse(''),
    padding: _cryptoJs["default"].pad.Pkcs7,
    mode: _cryptoJs["default"].mode.CBC
  });

  return cipher.toString();
}

function decodeAES128(str, key) {
  var input = str.startsWith('${AES}') ? str.substring(6) : str;

  var cipher = _cryptoJs["default"].AES.decrypt(input, _cryptoJs["default"].enc.Utf8.parse(key), {
    iv: _cryptoJs["default"].enc.Utf8.parse(''),
    padding: _cryptoJs["default"].pad.Pkcs7,
    mode: _cryptoJs["default"].mode.CBC
  });

  try {
    return cipher.toString(_cryptoJs["default"].enc.Utf8);
  } catch (error) {
    return str;
  }
}

var pagingRequestParse = function pagingRequestParse(params) {
  var page = parseInt((params === null || params === void 0 ? void 0 : params.page) || '0');
  var limit = parseInt((params === null || params === void 0 ? void 0 : params.limit) || '30');
  var sort = (params === null || params === void 0 ? void 0 : params.sort) || 'created_at';
  var dir = (params === null || params === void 0 ? void 0 : params.dir) || 'desc';
  return {
    page: page,
    limit: limit,
    sort: sort,
    dir: dir
  };
};

exports.pagingRequestParse = pagingRequestParse;

var pagingResponseParse = function pagingResponseParse(request, pageObject, fn) {
  var _a;

  var max_page = Math.ceil(pageObject.count / request.limit) - 1;
  return {
    items: (_a = pageObject.rows) === null || _a === void 0 ? void 0 : _a.map(function (v) {
      return fn ? fn((v === null || v === void 0 ? void 0 : v.dataValues) || v) : (v === null || v === void 0 ? void 0 : v.dataValues) || v;
    }),
    page: request.page,
    page_size: request.limit,
    max_page: max_page,
    has_prev: request.page > 0,
    has_next: request.page < max_page,
    total_elements: pageObject.count
  };
};

exports.pagingResponseParse = pagingResponseParse;

var wrappingFunciton = function wrappingFunciton(fn) {
  return function () {
    var params = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      params[_i] = arguments[_i];
    }

    return (0, _tslib.__awaiter)(this, void 0, void 0, function () {
      var sequlize, lastField, isOutTransaction, transaction, _a, repo, output, e_1;

      var _this = this;

      return (0, _tslib.__generator)(this, function (_b) {
        switch (_b.label) {
          case 0:
            sequlize = (0, _database.getSequelize)();
            if (!sequlize) return [2
            /*return*/
            ];
            lastField = params.length > 0 && params[params.length - 1];
            isOutTransaction = lastField instanceof _sequelize.Transaction;
            if (!isOutTransaction) return [3
            /*break*/
            , 1];
            _a = lastField;
            return [3
            /*break*/
            , 3];

          case 1:
            return [4
            /*yield*/
            , sequlize.transaction()];

          case 2:
            _a = _b.sent();
            _b.label = 3;

          case 3:
            transaction = _a;
            _b.label = 4;

          case 4:
            _b.trys.push([4, 8,, 11]);

            repo = {
              findAll: function findAll(model, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, output, result, _loop_1, _i, _a, field;

                  return (0, _tslib.__generator)(this, function (_b) {
                    switch (_b.label) {
                      case 0:
                        fields = (0, _database.getSecureFields)(model.tableName);
                        return [4
                        /*yield*/
                        , model.findAll((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          transaction: transaction
                        }))];

                      case 1:
                        output = _b.sent();
                        result = output;

                        _loop_1 = function _loop_1(field) {
                          result = result === null || result === void 0 ? void 0 : result.map(function (row) {
                            var _a, _b;

                            if ((_b = (_a = row === null || row === void 0 ? void 0 : row.getDataValue(field)) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, '${AES}')) {
                              row.setDataValue(field, decodeAES128(row.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                              if (model.rawAttributes[field].type.key === 'JSON') {
                                try {
                                  row.setDataValue(field, JSON.parse(row.getDataValue(field)));
                                } catch (error) {}
                              }
                            }

                            return row;
                          });
                        };

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];

                          _loop_1(field);
                        }

                        return [2
                        /*return*/
                        , result];
                    }
                  });
                });
              },
              findOne: function findOne(model, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var target, user, fields, output, _i, _a, field;

                  var _b, _c;

                  return (0, _tslib.__generator)(this, function (_d) {
                    switch (_d.label) {
                      case 0:
                        target = args.exportTo;
                        user = args.user;
                        delete args.exportTo;
                        delete args.user;
                        fields = (0, _database.getSecureFields)(model.tableName);
                        return [4
                        /*yield*/
                        , model.findOne(!target ? (0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          transaction: transaction
                        }) : target === null || target === void 0 ? void 0 : target.middleware((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          transaction: transaction
                        }), user))];

                      case 1:
                        output = _d.sent();

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];
                          if (!((_c = (_b = output === null || output === void 0 ? void 0 : output.getDataValue(field)) === null || _b === void 0 ? void 0 : _b.startsWith) === null || _c === void 0 ? void 0 : _c.call(_b, '${AES}'))) continue;
                          output.setDataValue(field, decodeAES128(output.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                          if (model.rawAttributes[field].type.key === 'JSON') {
                            try {
                              output.setDataValue(field, JSON.parse(output.getDataValue(field)));
                            } catch (error) {}
                          }
                        }

                        return [2
                        /*return*/
                        , output];
                    }
                  });
                });
              },
              findAndCountAll: function findAndCountAll(model, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, output, _loop_2, _i, _a, field;

                  var _b;

                  return (0, _tslib.__generator)(this, function (_c) {
                    switch (_c.label) {
                      case 0:
                        fields = (0, _database.getSecureFields)(model.tableName);
                        return [4
                        /*yield*/
                        , model.findAndCountAll((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          transaction: transaction
                        }))];

                      case 1:
                        output = _c.sent();

                        _loop_2 = function _loop_2(field) {
                          output.rows = (_b = output.rows) === null || _b === void 0 ? void 0 : _b.map(function (row) {
                            var _a, _b;

                            if ((_b = (_a = row.getDataValue(field)) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, '${AES}')) {
                              row.setDataValue(field, decodeAES128(row.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                              if (model.rawAttributes[field].type.key === 'JSON') {
                                try {
                                  row.setDataValue(field, JSON.parse(row.getDataValue(field)));
                                } catch (error) {}
                              }
                            }

                            return row;
                          });
                        };

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];

                          _loop_2(field);
                        }

                        return [2
                        /*return*/
                        , output];
                    }
                  });
                });
              },
              getObjects: function getObjects(model, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, isPaging, page, target, user, options, output_1, _loop_3, _i, _a, field, options, output_2, _loop_4, _b, _c, field;

                  var _d, _e;

                  return (0, _tslib.__generator)(this, function (_f) {
                    switch (_f.label) {
                      case 0:
                        fields = (0, _database.getSecureFields)(String(model.tableName));
                        isPaging = !!args.pagable;
                        page = pagingRequestParse(args.pagable);
                        target = args.exportTo;
                        user = args.user;
                        delete args.pagable;
                        delete args.exportTo;
                        delete args.user;
                        if (!isPaging) return [3
                        /*break*/
                        , 2];
                        options = target === null || target === void 0 ? void 0 : target.middleware((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          limit: page.limit,
                          offset: page.page * page.limit,
                          order: args.order || [[page.sort, page.dir]]
                        }), user);
                        return [4
                        /*yield*/
                        , model.findAndCountAll(options || (0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          limit: page.limit,
                          offset: page.page * page.limit,
                          order: args.order || [[page.sort, page.dir]]
                        }))];

                      case 1:
                        output_1 = _f.sent();

                        _loop_3 = function _loop_3(field) {
                          output_1.rows = (_d = output_1.rows) === null || _d === void 0 ? void 0 : _d.map(function (row) {
                            var _a, _b;

                            if ((_b = (_a = row.getDataValue(field)) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, '${AES}')) {
                              row.setDataValue(field, decodeAES128(row.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                              if (model.rawAttributes[field].type.key === 'JSON') {
                                try {
                                  row.setDataValue(field, JSON.parse(row.getDataValue(field)));
                                } catch (error) {}
                              }
                            }

                            return row;
                          });
                        };

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];

                          _loop_3(field);
                        }

                        return [2
                        /*return*/
                        , pagingResponseParse(page, output_1, (target === null || target === void 0 ? void 0 : target.map) || function (o) {
                          return o;
                        })];

                      case 2:
                        options = target === null || target === void 0 ? void 0 : target.middleware((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                          limit: args.limit,
                          order: args.order || [(_e = model.primaryKeyAttributes) === null || _e === void 0 ? void 0 : _e.map(function (attr) {
                            return [attr, 'desc'];
                          })]
                        }), user);
                        return [4
                        /*yield*/
                        , model.findAll(options)];

                      case 3:
                        output_2 = _f.sent();

                        _loop_4 = function _loop_4(field) {
                          // @ts-ignore
                          output_2 = output_2 === null || output_2 === void 0 ? void 0 : output_2.map(function (row) {
                            var _a, _b;

                            if ((_b = (_a = row.getDataValue(field)) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, '${AES}')) {
                              row.setDataValue(field, decodeAES128(row.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                              if (model.rawAttributes[field].type.key === 'JSON') {
                                try {
                                  row.setDataValue(field, JSON.parse(row.getDataValue(field)));
                                } catch (error) {}
                              }
                            }

                            return row;
                          });
                        };

                        for (_b = 0, _c = fields !== null && fields !== void 0 ? fields : []; _b < _c.length; _b++) {
                          field = _c[_b];

                          _loop_4(field);
                        }

                        return [2
                        /*return*/
                        , target === null || target === void 0 ? void 0 : target.collect(output_2)];
                    }
                  });
                });
              },
              count: function count(model, args) {
                return model.count((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                  transaction: transaction
                }));
              },
              create: function create(model, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, args2, _i, _a, field, output, _b, _c, field;

                  var _d, _e, _f, _g;

                  return (0, _tslib.__generator)(this, function (_h) {
                    switch (_h.label) {
                      case 0:
                        fields = (0, _database.getSecureFields)(model === null || model === void 0 ? void 0 : model.tableName);
                        args2 = (0, _tslib.__assign)({}, args);

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];
                          if (args2[field] === null || args2[field] === undefined || ((_e = (_d = args2[field]) === null || _d === void 0 ? void 0 : _d.startsWith) === null || _e === void 0 ? void 0 : _e.call(_d, '${AES}'))) continue; // @ts-ignore

                          if (model.rawAttributes[field].type.key === 'JSON') {
                            args2[field] = '${AES}' + encodeAES128(JSON.stringify(args2[field]), (0, _database.getSecureKey)());
                          } else {
                            args2[field] = '${AES}' + encodeAES128(args2[field], (0, _database.getSecureKey)());
                          }
                        }

                        return [4
                        /*yield*/
                        , model.create(args2, {
                          transaction: transaction
                        })];

                      case 1:
                        output = _h.sent();

                        for (_b = 0, _c = fields !== null && fields !== void 0 ? fields : []; _b < _c.length; _b++) {
                          field = _c[_b];
                          if ((output === null || output === void 0 ? void 0 : output.getDataValue(field)) === null || (output === null || output === void 0 ? void 0 : output.getDataValue(field)) === undefined || !((_g = (_f = output === null || output === void 0 ? void 0 : output.getDataValue(field)) === null || _f === void 0 ? void 0 : _f.startsWith) === null || _g === void 0 ? void 0 : _g.call(_f, '${AES}'))) continue;
                          output.setDataValue(field, decodeAES128(output.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                          if (model.rawAttributes[field].type.key === 'JSON') {
                            try {
                              output.setDataValue(field, JSON.parse(output.getDataValue(field)));
                            } catch (error) {}
                          }
                        }

                        return [2
                        /*return*/
                        , output];
                    }
                  });
                });
              },
              save: function save(model) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, _i, _a, field, output, _b, _c, field;

                  var _d, _e, _f, _g;

                  return (0, _tslib.__generator)(this, function (_h) {
                    switch (_h.label) {
                      case 0:
                        fields = (0, _database.getSecureFields)(model.constructor.name);

                        for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                          field = _a[_i];
                          if (model.getDataValue === null || model.getDataValue === undefined || ((_e = (_d = model.getDataValue(field)) === null || _d === void 0 ? void 0 : _d.startsWith) === null || _e === void 0 ? void 0 : _e.call(_d, '${AES}'))) continue; // @ts-ignore

                          if (model.rawAttributes[field].type.key === 'JSON') {
                            model.setDataValue(field, '${AES}' + encodeAES128(JSON.stringify(model.getDataValue(field)), (0, _database.getSecureKey)()));
                          } else {
                            model.setDataValue(field, '${AES}' + encodeAES128(model.getDataValue(field), (0, _database.getSecureKey)()));
                          }
                        }

                        return [4
                        /*yield*/
                        , model.save({
                          transaction: transaction
                        })];

                      case 1:
                        output = _h.sent();

                        for (_b = 0, _c = fields !== null && fields !== void 0 ? fields : []; _b < _c.length; _b++) {
                          field = _c[_b];
                          if ((output === null || output === void 0 ? void 0 : output.getDataValue(field)) === null || (output === null || output === void 0 ? void 0 : output.getDataValue(field)) === undefined || !((_g = (_f = output === null || output === void 0 ? void 0 : output.getDataValue(field)) === null || _f === void 0 ? void 0 : _f.startsWith) === null || _g === void 0 ? void 0 : _g.call(_f, '${AES}'))) continue;
                          output.setDataValue(field, decodeAES128(output.getDataValue(field), (0, _database.getSecureKey)())); // @ts-ignore

                          if (model.rawAttributes[field].type.key === 'JSON') {
                            try {
                              output.setDataValue(field, JSON.parse(output.getDataValue(field)));
                            } catch (error) {}
                          }
                        }

                        return [2
                        /*return*/
                        , output];
                    }
                  });
                });
              },
              destroy: function destroy(model) {
                return model.destroy({
                  transaction: transaction
                });
              },
              destroyAll: function destroyAll(model, args) {
                return model.destroy((0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                  transaction: transaction
                }));
              },
              updateAll: function updateAll(model, values, args) {
                return (0, _tslib.__awaiter)(_this, void 0, void 0, function () {
                  var fields, _i, _a, field;

                  var _b, _c;

                  return (0, _tslib.__generator)(this, function (_d) {
                    fields = (0, _database.getSecureFields)(model === null || model === void 0 ? void 0 : model.tableName);

                    for (_i = 0, _a = fields !== null && fields !== void 0 ? fields : []; _i < _a.length; _i++) {
                      field = _a[_i];
                      if ((_c = (_b = values[field]) === null || _b === void 0 ? void 0 : _b.startsWith) === null || _c === void 0 ? void 0 : _c.call(_b, '${AES}')) continue;
                      values[field] = '${AES}' + encodeAES128(values[field], (0, _database.getSecureKey)());
                    } // @ts-ignore


                    return [2
                    /*return*/
                    , model.update(values, (0, _tslib.__assign)((0, _tslib.__assign)({}, args), {
                      transaction: transaction
                    }))];
                  });
                });
              }
            };
            return [4
            /*yield*/
            , fn(repo, params, {
              transaction: transaction
            })];

          case 5:
            output = _b.sent();
            if (!!isOutTransaction) return [3
            /*break*/
            , 7];
            return [4
            /*yield*/
            , transaction.commit()];

          case 6:
            _b.sent();

            _b.label = 7;

          case 7:
            return [2
            /*return*/
            , output];

          case 8:
            e_1 = _b.sent();
            if (!!isOutTransaction) return [3
            /*break*/
            , 10];
            return [4
            /*yield*/
            , transaction.rollback()];

          case 9:
            _b.sent();

            _b.label = 10;

          case 10:
            throw e_1;

          case 11:
            return [2
            /*return*/
            ];
        }
      });
    });
  };
};

var createService = function createService(properties) {
  var methods = {};
  Object.keys(properties).forEach(function (key) {
    // @ts-ignore
    methods[key] = wrappingFunciton(properties[key]);
  });
  return methods;
};

exports.createService = createService;