"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.socketSendAll = exports.socketSendTo = exports.getSocketIO = exports.socketInit = exports.createSocket = void 0;

var _tslib = require("tslib");

var config = {
  container: undefined,
  listeners: []
};

var createSocket = function createSocket(request, execute) {
  config.listeners.push({
    request: request,
    execute: execute
  });
};

exports.createSocket = createSocket;

var socketInit = function socketInit(server, options, authOption) {
  if (config.container) return;

  var io = require('socket.io')(server);

  if (options.adapter) {
    io.adapter(options.adapter);
  }

  if (authOption) {
    var auth = authOption;

    if (authOption.jwt_secret && authOption.handler) {
      var jwtAuth = require('socketio-jwt-auth');

      var handler_1 = auth.handler();
      var jwtHandler = jwtAuth.authenticate({
        secret: auth.jwt_secret,
        algorithm: 'HS256'
      }, function (payload, done) {
        return (0, _tslib.__awaiter)(this, void 0, void 0, function () {
          var user, e_1;
          return (0, _tslib.__generator)(this, function (_a) {
            switch (_a.label) {
              case 0:
                _a.trys.push([0, 2,, 3]);

                return [4
                /*yield*/
                , handler_1(payload)];

              case 1:
                user = _a.sent();

                if (user) {
                  return [2
                  /*return*/
                  , done(null, user)];
                } else {
                  return [2
                  /*return*/
                  , done()];
                }

                return [3
                /*break*/
                , 3];

              case 2:
                e_1 = _a.sent();
                return [2
                /*return*/
                , done(e_1)];

              case 3:
                return [2
                /*return*/
                ];
            }
          });
        });
      });
      io.use(jwtHandler);
    }
  }

  io.on('connection', function (socket) {
    var user = socket.request.user;

    if (authOption) {
      // now you can access user info through socket.request.user
      // socket.request.user.logged_in will be set to true if the user was authenticated
      socket.emit('success', {
        message: 'success logged in!',
        user: socket.request.user
      });
    }

    config.listeners.map(function (item) {
      var _a;

      var r = (_a = item.request.roles) === null || _a === void 0 ? void 0 : _a.filter(function (v) {
        var _a;

        return (_a = user === null || user === void 0 ? void 0 : user.roles) === null || _a === void 0 ? void 0 : _a.includes(v);
      });

      if ((r === null || r === void 0 ? void 0 : r.length) > 0 || item.request.roles === undefined) {
        socket.on(item.request.path, function (body) {
          return (0, _tslib.__awaiter)(void 0, void 0, void 0, function () {
            return (0, _tslib.__generator)(this, function (_a) {
              switch (_a.label) {
                case 0:
                  return [4
                  /*yield*/
                  , item.execute(socket, {
                    body: body,
                    user: user
                  })];

                case 1:
                  _a.sent();

                  return [2
                  /*return*/
                  ];
              }
            });
          });
        });
      }
    });
  });
  config.container = io;
  return io;
};

exports.socketInit = socketInit;

var getSocketIO = function getSocketIO() {
  return config.container;
};

exports.getSocketIO = getSocketIO;

var socketSendTo = function socketSendTo(to, event, data) {
  var _a;

  return (_a = config.container) === null || _a === void 0 ? void 0 : _a.to(to).emit(event, data);
};

exports.socketSendTo = socketSendTo;

var socketSendAll = function socketSendAll(event, data) {
  var _a;

  return (_a = config.container) === null || _a === void 0 ? void 0 : _a.emit(event, data);
};

exports.socketSendAll = socketSendAll;