"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.run = void 0;

var _tslib = require("tslib");

var _fs = _interopRequireDefault(require("fs"));

var _express = _interopRequireDefault(require("express"));

var _expressFileupload = _interopRequireDefault(require("express-fileupload"));

var _cors = _interopRequireDefault(require("cors"));

var _passport = _interopRequireDefault(require("passport"));

var _http = _interopRequireDefault(require("http"));

var _database = require("./lib/database");

var _swagger = _interopRequireDefault(require("./lib/swagger"));

var _route = require("./lib/route");

var _socket = require("./lib/socket");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var run = function run(dirname, options) {
  var loadPath = function loadPath(path, subPath) {
    return (0, _tslib.__awaiter)(void 0, void 0, void 0, function () {
      var files, _i, files_1, file;

      return (0, _tslib.__generator)(this, function (_a) {
        switch (_a.label) {
          case 0:
            return [4
            /*yield*/
            , _fs["default"].readdirSync(path + subPath, {
              withFileTypes: true
            })];

          case 1:
            files = _a.sent();
            _i = 0, files_1 = files;
            _a.label = 2;

          case 2:
            if (!(_i < files_1.length)) return [3
            /*break*/
            , 6];
            file = files_1[_i];
            if (!file.isDirectory()) return [3
            /*break*/
            , 4];
            return [4
            /*yield*/
            , loadPath(path, subPath + '/' + file.name)];

          case 3:
            _a.sent();

            return [3
            /*break*/
            , 5];

          case 4:
            if (file.name.match(/\.js$/) !== null) {
              // eslint-disable-next-line no-undef
              require(dirname + '/app' + subPath + '/' + file.name);
            }

            _a.label = 5;

          case 5:
            _i++;
            return [3
            /*break*/
            , 2];

          case 6:
            return [2
            /*return*/
            ];
        }
      });
    });
  };

  var initServer = function initServer() {
    return (0, _tslib.__awaiter)(void 0, void 0, void 0, function () {
      var dbContainer, app, _a, _b, _c, _d, _e;

      return (0, _tslib.__generator)(this, function (_f) {
        switch (_f.label) {
          case 0:
            return [4
            /*yield*/
            , (0, _database.dbInit)(options.database)];

          case 1:
            dbContainer = _f.sent();
            return [4
            /*yield*/
            , loadPath(dirname + '/app', '')];

          case 2:
            _f.sent();

            return [4
            /*yield*/
            , (0, _database.dbAssociate)()];

          case 3:
            _f.sent();

            app = (0, _express["default"])();
            app.use((0, _cors["default"])({
              origin: '*',
              methods: ['GET', 'PUT', 'POST', 'DELETE', 'HEAD', 'OPTIONS'],
              credentials: true
            }));
            app.use(function (req, res, next) {
              var _a;

              if ((_a = options.rawUris) === null || _a === void 0 ? void 0 : _a.includes(req.originalUrl)) {
                _express["default"].raw({
                  type: 'application/json'
                })(req, res, next);
              } else {
                _express["default"].json({
                  limit: '500mb'
                })(req, res, next);
              }
            });
            app.use(_express["default"].urlencoded({
              limit: '500mb',
              extended: false
            }));
            app.use(_passport["default"].initialize({
              userProperty: ''
            }));
            app.use((0, _expressFileupload["default"])({}));
            if (!options.prefix) return [3
            /*break*/
            , 5];
            _b = (_a = app).use;
            _c = ["/" + options.prefix];
            return [4
            /*yield*/
            , (0, _route.installRoutes)(options.auth)];

          case 4:
            _b.apply(_a, _c.concat([_f.sent()]));

            return [3
            /*break*/
            , 7];

          case 5:
            _e = (_d = app).use;
            return [4
            /*yield*/
            , (0, _route.installRoutes)(options.auth)];

          case 6:
            _e.apply(_d, [_f.sent()]);

            _f.label = 7;

          case 7:
            (0, _swagger["default"])(app, options.swagger, options.prefix);

            if (dbContainer) {
              app.use('/sync', function (req, res) {
                return (0, _tslib.__awaiter)(void 0, void 0, void 0, function () {
                  var ip;
                  return (0, _tslib.__generator)(this, function (_a) {
                    switch (_a.label) {
                      case 0:
                        ip = req.headers['x-forwarded-for'] || req.connection.remoteAddress || '';
                        if (!['::1', 'localhost', '127.0.0.1', '::ffff:127.0.0.1'].includes(ip)) return [3
                        /*break*/
                        , 2];
                        return [4
                        /*yield*/
                        , dbContainer.sync({
                          alter: {
                            drop: false
                          }
                        })];

                      case 1:
                        _a.sent();

                        res.type('text/plain');
                        res.status(200);
                        res.send('db-sync');
                        return [3
                        /*break*/
                        , 3];

                      case 2:
                        res.type('text/plain');
                        res.status(404);
                        res.send('404 - Not Found');
                        _a.label = 3;

                      case 3:
                        return [2
                        /*return*/
                        ];
                    }
                  });
                });
              });
            }

            app.use('/', function (_req, res) {
              res.type('text/plain');
              res.status(200);
              res.send(options.swagger.name || '@actbase/node-server');
            });
            app.use(function (_req, res) {
              res.type('text/plain');
              res.status(404);
              res.send('404 - Not Found');
            });
            return [2
            /*return*/
            , {
              app: app
            }];
        }
      });
    });
  };

  initServer().then(function (_a) {
    var app = _a.app;

    var server = _http["default"].createServer(app);

    if (options.socket) {
      (0, _socket.socketInit)(server, options.socket, options.auth);
    }

    var port = options.port || 3100;
    server.listen(port, function () {
      var _a;

      console.log("Server is running on " + port + " port.  http://localhost:" + port);
      (_a = options === null || options === void 0 ? void 0 : options.listener) === null || _a === void 0 ? void 0 : _a.call(options);
    });
  })["catch"](console.warn);
};

exports.run = run;
var _default = run;
exports["default"] = _default;